import { registerPlugin } from "@capacitor/core";
import type { SocketsPlugin } from "./definitions";
import mitt, { Emitter } from "mitt";

const Sockets = registerPlugin<SocketsPlugin>("Sockets");

type SocketEvents = {
    message: string;
    state: string;
};

class SocketManager {
    private static instance: SocketManager;
    private socketsMap: Map<string, Socket> = new Map();

    private constructor() {
        this.addGlobalListeners();
    }

    public static getInstance() {
        if (!SocketManager.instance) {
            SocketManager.instance = new SocketManager();
        }
        return SocketManager.instance;
    }

    public registerSocket(socket: Socket) {
        this.socketsMap.set(socket.id, socket);
    }

    public unregisterSocket(socket: Socket) {
        this.socketsMap.delete(socket.id);
    }

    private addGlobalListeners() {
        Sockets.addListener("state", (data) => {
            let socket = this.socketsMap.get(data.id);
            if (socket) {
                socket.didChangeState(data);
            }
        });

        Sockets.addListener("message", (data) => {
            let socket = this.socketsMap.get(data.id);
            if (socket) {
                socket.didReceiveMessage(data);
            }
        });
    }
}

export default class Socket {
    id: string;
    private emitter: Emitter<SocketEvents>;

    constructor(config: { id: string }) {
        console.log(config);
        this.id = config.id;
        this.emitter = mitt<SocketEvents>();
        this.create();
        SocketManager.getInstance().registerSocket(this);
    }

    on(event: keyof SocketEvents, handler: (data: any) => void) {
        this.emitter.on(event, handler);
    }

    async create() {
        await Sockets.create({
            id: this.id
        });
    }

    async connect(config: { host: string; port: number; useTLS?: boolean; acceptInvalidCertificates?: boolean; delimiter?: string }) {
        await Sockets.connect({
            id: this.id,
            host: config.host,
            port: config.port,
            useTLS: config.useTLS ?? false,
            acceptInvalidCertificates: config.acceptInvalidCertificates ?? false,
            delimiter: config.delimiter ?? "\r\n"
        });
    }

    async disconnect() {
        await Sockets.disconnect({
            id: this.id
        });
    }

    async close() {
        await this.disconnect();
        SocketManager.getInstance().unregisterSocket(this);
    }

    async send(message: string) {
        await Sockets.send({
            id: this.id,
            message: message
        });
    }

    async setNotificationText(config: { notificationText: string }) {
        await Sockets.setNotificationText({
            notificationText: config.notificationText
        });
    }

    didChangeState(data: { state: string }) {
        this.emitter.emit("state", data.state);
    }

    didReceiveMessage(data: { message: string }) {
        this.emitter.emit("message", data.message);
    }
}